/***********************************************************************
*
* Copyright (c) 2021-2023 Barbara Geller and Ansel Sermersheim
*
* This example is free software, released under the BSD 2-Clause license.
* For license details refer to LICENSE provided with this project.
*
* CopperSpice is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
* https://opensource.org/licenses/BSD-2-Clause
*
***********************************************************************/

#include <QtCore>
#include <QtGui>

class MainWindow : public QWidget
{
 public:
   MainWindow();

 private:
   QLineEdit *textEdit;

   QLineEdit *regex_A1;
   QLineEdit *regex_A2;
   QLineEdit *regex_B;
   QLineEdit *regex_C;

   void parseText();
};

MainWindow::MainWindow()
{
   setMinimumSize(700, 400);

   QLabel *textLabel = new QLabel("Text: ");

   textEdit = new QLineEdit();
   textEdit->setText("If people never did silly things, nothing intelligent would ever get done.");

   QLabel *label_A1 = new QLabel();
   label_A1->setText("First match, regex: \\\\w*o\\\\w");
   label_A1->setMinimumWidth(170);
   regex_A1 = new QLineEdit();

   QLabel *label_A2 = new QLabel();
   label_A2->setText("Every match, regex: \\\\w*o\\\\w");
   label_A2->setMinimumWidth(170);
   regex_A2 = new QLineEdit();

   QLabel *label_B = new QLabel();
   label_B->setText("Every match, regex: e(\\\\w*)");
   label_B->setMinimumWidth(170);
   regex_B = new QLineEdit();

   QLabel *label_C = new QLabel();
   label_C->setText("Replace, regex: \\\\b\\\\w{3}\\\\b");
   label_C->setMinimumWidth(170);
   regex_C = new QLineEdit();

   // push buttons
   QPushButton *pb_run = new QPushButton();
   pb_run->setText("Run QRegularExpression");

   QPushButton *pb_close = new QPushButton();
   pb_close->setText("Close");

   // layout
   QGroupBox *box1 = new QGroupBox("Example 1");
   QGroupBox *box2 = new QGroupBox("Example 2");
   QGroupBox *box3 = new QGroupBox("Example 3");

   setStyleSheet("::title{color:blue}");

   QFormLayout *form1 = new QFormLayout();
   form1->addRow(label_A1, regex_A1);
   form1->addRow(label_A2, regex_A2);
   box1->setLayout(form1);

   QFormLayout *form2 = new QFormLayout();
   form2->addRow(label_B, regex_B);
   box2->setLayout(form2);

   QFormLayout *form3 = new QFormLayout();
   form3->addRow(label_C, regex_C);
   box3->setLayout(form3);

   QFormLayout *formMain = new QFormLayout();
   formMain->setHorizontalSpacing(15);
   formMain->setVerticalSpacing(25);

   formMain->addRow(textLabel, textEdit);
   formMain->addRow(nullptr, box1);
   formMain->addRow(nullptr, box2);
   formMain->addRow(nullptr, box3);

   QHBoxLayout *boxMain = new QHBoxLayout();
   boxMain->addStretch();
   boxMain->addWidget(pb_run);
   boxMain->addSpacing(20);
   boxMain->addWidget(pb_close);
   boxMain->addStretch();

   QVBoxLayout *layoutMain = new QVBoxLayout(this);
   layoutMain->setContentsMargins(20, 40, 30, 15);
   layoutMain->addLayout(formMain);
   layoutMain->addStretch();
   layoutMain->addLayout(boxMain);

   connect(pb_run,   &QPushButton::clicked, this, &MainWindow::parseText);
   connect(pb_close, &QPushButton::clicked, this, &QWidget::close);

   parseText();
}

void MainWindow::parseText()
{
   QString text = textEdit->text();

   {
      QRegularExpression regExp("\\w*o\\w*");
      QRegularExpressionMatch match = regExp.match(text);

      QString result = "No match";

      if (match.hasMatch()) {
         result = match.captured(0);
      }

      regex_A1->setText(result);

      //
      QList<QRegularExpressionMatch> matchList;
      matchList = regExp.globalMatch(text);

      QStringList list;
      for (auto &item : matchList) {
         list.append(item.captured(0));
      }

      regex_A2->setText(list.join(", "));
   }

   {
      QRegularExpression regExp("e(\\w*)");

      QString result = "No match";

      QList<QRegularExpressionMatch> matchList;
      matchList = regExp.globalMatch(text);

      QStringList list;
      for (auto &item : matchList) {
         list.append(item.captured(1));
      }

      regex_B->setText(list.join(", "));
   }

   {
      QRegularExpression regExp("\\b\\w{3}\\b");

      QString result = text;
      result.replace(regExp, "***");

      regex_C->setText(result);
   }
}

int main(int argc, char *argv[])
{
   QApplication app(argc, argv);

   MainWindow *window = new MainWindow();
   window->show();

   return app.exec();
}
